package eu.mhsl.craftattack.spawn.appliances.whitelist;

import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.util.text.DisconnectInfo;
import org.apache.http.client.utils.URIBuilder;
import org.bukkit.event.Listener;
import org.bukkit.event.player.AsyncPlayerPreLoginEvent;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

public class Whitelist extends Appliance {
    private record UserData(UUID uuid, String username, String firstname, String lastname) {}

    private final URI apiEndpoint = URI.create(Objects.requireNonNull(localConfig().getString("api")));

    public Whitelist() {
        super("whitelist");
    }

    public void login(AsyncPlayerPreLoginEvent login) throws DisconnectInfo.Throwable {
        try {
            UserData data = this.fetchUserData(login.getUniqueId());

            if(!data.username.equalsIgnoreCase(login.getName()))
                throw new DisconnectInfo.Throwable(
                        "Nutzername geändert",
                        String.format("Der Name '%s' stimmt nicht mit '%s' überein.", data.username, login.getName()),
                        "Bitte kontaktiere einen Admin, um Deine Anmeldedaten zu aktualisieren!",
                        login.getUniqueId()
                );

        } catch (DisconnectInfo.Throwable e) {
            throw e;
        } catch (Exception e) {
            throw new DisconnectInfo.Throwable(
                    "Interner Serverfehler",
                    "Deine Zugangsdaten konnten nicht abgerufen werden.",
                    "Versuche es später erneut oder kontaktiere einen Admin!",
                    login.getUniqueId()
            );
        }
    }

    private UserData fetchUserData(UUID uuid) throws DisconnectInfo.Throwable {
        URIBuilder uriBuilder = new URIBuilder(apiEndpoint);
        uriBuilder.addParameter("uuid", uuid.toString());

        try {
            HttpClient client = HttpClient.newHttpClient();
            HttpRequest httpRequest = HttpRequest.newBuilder()
                    .uri(uriBuilder.build())
                    .header("Content-Type", "application/json")
                    .GET()
                    .build();

            HttpResponse<String> httpResponse = client.send(httpRequest, HttpResponse.BodyHandlers.ofString());

            if(httpResponse.statusCode() == 400)
                throw new DisconnectInfo.Throwable(
                        "Nicht angemeldet",
                        "Du bist derzeit nicht als Teilnehmer des CraftAttack-Projektes registriert!",
                        "Melde Dich bei einem Admin für eine nachträgliche Anmeldung.",
                        uuid
                );

            return new Gson().fromJson(httpResponse.body(), UserData.class);

        } catch (IOException | InterruptedException | URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    protected @NotNull List<Listener> eventHandlers() {
        return List.of(
                new PlayerJoinListener()
        );
    }
}
